==> v1.0.6.zip: portfolio/web/src/components/Project/ProjectForm/ProjectForm.tsx <==
import { useEffect, useMemo, useRef, useState } from 'react'

import { mdiCalendar, mdiDelete, mdiFormatTitle, mdiLinkVariant } from '@mdi/js'
import Icon from '@mdi/react'
import Link from '@tiptap/extension-link'
import Underline from '@tiptap/extension-underline'
import { useEditor } from '@tiptap/react'
import StarterKit from '@tiptap/starter-kit'
import { Meta, UploadResult } from '@uppy/core'
import { format, isAfter, startOfToday } from 'date-fns'
import type {
  EditProjectById,
  FindTags,
  UpdateProjectInput,
} from 'types/graphql'

import type { RWGqlError } from '@redwoodjs/forms'
import { Form, FieldError, Label, TextField, Submit } from '@redwoodjs/forms'
import { toast } from '@redwoodjs/web/toast'

import DatePicker from 'src/components/DatePicker'
import FormTextList from 'src/components/FormTextList'
import RichTextEditor from 'src/components/RichTextEditor/RichTextEditor'
import TagsSelectorCell from 'src/components/Tag/TagsSelectorCell'
import Uploader from 'src/components/Uploader'
import { batchDelete } from 'src/lib/tus'

type FormProject = NonNullable<EditProjectById['project']>

interface ProjectFormProps {
  project?: EditProjectById['project']
  onSave: (data: UpdateProjectInput, id?: FormProject['id']) => void
  error: RWGqlError
  loading: boolean
}

const ProjectForm = (props: ProjectFormProps) => {
  const today = startOfToday()

  const descEditor = useEditor({
    extensions: [
      StarterKit,
      Underline,
      Link.configure({
        openOnClick: false,
        autolink: true,
        linkOnPaste: true,
        defaultProtocol: 'https',
      }),
    ],
    content: props.project?.description || '',
  })

  const [links, setLinks] = useState<string[]>(props.project?.links || [])
  const [linkErrors, setLinkErrors] = useState<boolean[]>([])
  const [pickerVisible, setPickerVisible] = useState<boolean>(false)
  const [date, setDate] = useState<Date>(
    props.project?.date ? new Date(props.project.date) : today
  )
  const [month, setMonth] = useState<string>(format(today, 'MMMM yyyy'))
  const [fileIds, setFileIds] = useState<string[]>(props.project?.images || [])
  const [selectedTags, setSelectedTags] = useState<FindTags['tags']>(
    props.project?.tags || []
  )
  const [appendUploader, setAppendUploader] = useState<boolean>(false)
  const [toDelete, setToDelete] = useState<string[]>([])

  const onUploadComplete = (
    result: UploadResult<Meta, Record<string, never>>
  ) => {
    setFileIds(
      appendUploader
        ? [...fileIds, ...result.successful.map((file) => file.uploadURL)]
        : result.successful.map((file) => file.uploadURL)
    )
    setAppendUploader(false)
  }

  const urlRegex = useMemo(
    () =>
      /^(?:(?:(?:https?|ftp):)?\/\/)(?:\S+(?::\S*)?@)?(?:(?!(?:10|127)(?:\.\d{1,3}){3})(?!(?:169\.254|192\.168)(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)(?:\.(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)*(?:\.(?:[a-z\u00a1-\uffff]{2,})))(?::\d{2,5})?(?:[/?#]\S*)?$/i,
    []
  )

  useEffect(() => {
    setLinkErrors(links.map((link) => link.length > 0 && !urlRegex.test(link)))
  }, [links, urlRegex])

  const onSubmit = (data: FormProject) => {
    const errorsExist = linkErrors.indexOf(true) !== -1
    const errorCount = linkErrors.filter((val) => val).length
    const emptyCount = links.filter((val) => val.trim().length === 0).length

    if (errorsExist) return toast.error(`${errorCount} links invalid`)
    if (emptyCount > 0) return toast.error(`${emptyCount} links empty`)

    batchDelete(toDelete)

    props.onSave(
      {
        title: data.title,
        description: descEditor.getHTML(),
        date: date.toISOString(),
        links: links.filter((link) => link.trim().length > 0),
        images: fileIds,
        tags: selectedTags.map((tag) => tag.id),
        removeTags: props.project?.tags
          .filter((tag) => !selectedTags.some((st) => st.id === tag.id))
          .map((tag) => tag.id),
      },
      props?.project?.id
    )
  }

  const titleRef = useRef<HTMLInputElement>(null)
  useEffect(() => titleRef.current?.focus(), [])

  return (
    <Form<FormProject>
      onSubmit={onSubmit}
      error={props.error}
      className="space-y-2 w-80"
    >
      <Label name="title" className="form-control w-full">
        <Label
          name="title"
          className="input input-bordered flex items-center gap-2"
          errorClassName="input input-bordered flex items-center gap-2 input-error"
        >
          <Label
            name="title"
            className="size-4 opacity-70"
            errorClassName="size-4 text-error"
          >
            <Icon path={mdiFormatTitle} />
          </Label>
          <TextField
            name="title"
            ref={titleRef}
            placeholder="Title"
            defaultValue={props.project?.title}
            className="w-full"
            validation={{
              required: {
                value: true,
                message: 'Required',
              },
            }}
          />
        </Label>
        <div className="label">
          <FieldError
            name="title"
            className="text-xs font-semibold text-error"
          />
        </div>
      </Label>

      <RichTextEditor editor={descEditor} />

      <div className="form-control w-full">
        <Label
          name="date"
          className="input input-bordered flex items-center gap-2"
          errorClassName="input input-bordered flex items-center gap-2 input-error"
          onClick={() => setPickerVisible(!pickerVisible)}
        >
          <Label
            name="date"
            className="size-4 opacity-70"
            errorClassName="size-4 text-error"
          >
            <Icon path={mdiCalendar} />
          </Label>
          <TextField
            name="date"
            ref={titleRef}
            placeholder="Date"
            value={format(date, 'PP')}
            className="w-full"
          />
        </Label>
      </div>

      {pickerVisible && (
        <div className="flex justify-center">
          <DatePicker
            date={date}
            setDate={setDate}
            month={month}
            setMonth={setMonth}
          />
        </div>
      )}

      <div className={`${!pickerVisible && 'pt-2'}`}>
        <FormTextList
          name="Links"
          hint="Short links are recommended"
          itemPlaceholder="URL"
          icon={mdiLinkVariant}
          list={links}
          errors={linkErrors}
          setList={setLinks}
        />
      </div>

      <TagsSelectorCell
        selectedTags={selectedTags}
        setSelectedTags={setSelectedTags}
      />

      <div className="py-2 space-y-2">
        <div className="flex space-x-2 justify-between items-center">
          <p className="font-semibold">Images</p>
          {fileIds.length > 0 &&
            (appendUploader ? (
              <button
                className="btn btn-sm"
                type="button"
                onClick={() => setAppendUploader(false)}
              >
                Nevermind
              </button>
            ) : (
              <button
                className="btn btn-sm"
                type="button"
                onClick={() => setAppendUploader(true)}
              >
                Upload More
              </button>
            ))}
        </div>
        {fileIds.length > 0 ? (
          <>
            {!appendUploader &&
              fileIds.map((fileId, i) => (
                <div key={i} className="flex justify-center">
                  <div className="card rounded-xl w-fit image-full image-full-no-overlay">
                    <figure>
                      <img src={fileId} alt={i.toString()} />
                    </figure>
                    <div className="card-body p-2 rounded-xl">
                      <div className="card-actions rounded-md justify-end">
                        <button
                          type="button"
                          className="btn btn-square btn-sm btn-error shadow-xl"
                          onClick={() => {
                            setToDelete([...toDelete, fileId])
                            setFileIds(fileIds.filter((id) => id !== fileId))
                          }}
                        >
                          <Icon path={mdiDelete} className="size-4" />
                        </button>
                      </div>
                    </div>
                  </div>
                </div>
              ))}
            {appendUploader && (
              <Uploader
                onComplete={onUploadComplete}
                width="20rem"
                height="30rem"
                className="flex justify-center"
                maxFiles={10}
                disabled={props.loading}
              />
            )}
          </>
        ) : (
          <Uploader
            onComplete={onUploadComplete}
            width="20rem"
            height="30rem"
            className="flex justify-center pt-3"
            maxFiles={10}
            disabled={props.loading}
          />
        )}
      </div>

      {isAfter(date, today) && (
        <div className="flex justify-center py-2">
          <p>Project will be marked as</p>
          <div className="ml-1 badge badge-info">planned</div>
        </div>
      )}

      <nav className="my-2 flex justify-center space-x-2">
        <Submit
          disabled={props.loading}
          className="btn btn-primary btn-sm uppercase"
        >
          Save
        </Submit>
      </nav>
    </Form>
  )
}

export default ProjectForm

==> v1.1.6-1.zip: portfolio/web/src/components/Project/ProjectForm/ProjectForm.tsx <==
import { useEffect, useMemo, useRef, useState } from 'react'

import { mdiCalendar, mdiDelete, mdiFormatTitle, mdiLinkVariant } from '@mdi/js'
import Icon from '@mdi/react'
import Link from '@tiptap/extension-link'
import Underline from '@tiptap/extension-underline'
import { useEditor } from '@tiptap/react'
import StarterKit from '@tiptap/starter-kit'
import { Meta, UploadResult } from '@uppy/core'
import { format, isAfter, startOfToday } from 'date-fns'
import type {
  EditProjectById,
  FindTags,
  UpdateProjectInput,
} from 'types/graphql'

import type { RWGqlError } from '@redwoodjs/forms'
import { Form, FieldError, Label, TextField, Submit } from '@redwoodjs/forms'
import { toast } from '@redwoodjs/web/toast'

import DatePicker from 'src/components/DatePicker'
import FormTextList from 'src/components/FormTextList'
import RichTextEditor from 'src/components/RichTextEditor/RichTextEditor'
import TagsSelectorCell from 'src/components/Tag/TagsSelectorCell'
import Uploader from 'src/components/Uploader'
import { batchDelete } from 'src/lib/tus'

type FormProject = NonNullable<EditProjectById['project']>

interface ProjectFormProps {
  project?: EditProjectById['project']
  onSave: (data: UpdateProjectInput, id?: FormProject['id']) => void
  error: RWGqlError
  loading: boolean
}

const ProjectForm = (props: ProjectFormProps) => {
  const today = startOfToday()

  const descEditor = useEditor({
    extensions: [
      StarterKit,
      Underline,
      Link.configure({
        openOnClick: false,
        autolink: true,
        linkOnPaste: true,
        defaultProtocol: 'https',
      }),
    ],
    content: props.project?.description || '',
  })

  const [links, setLinks] = useState<string[]>(props.project?.links || [])
  const [linkErrors, setLinkErrors] = useState<boolean[]>([])
  const [pickerVisible, setPickerVisible] = useState<boolean>(false)
  const [date, setDate] = useState<Date>(
    props.project?.date ? new Date(props.project.date) : today
  )
  const [month, setMonth] = useState<string>(format(today, 'MMMM yyyy'))
  const [fileIds, setFileIds] = useState<string[]>(props.project?.images || [])
  const [selectedTags, setSelectedTags] = useState<FindTags['tags']>(
    props.project?.tags || []
  )
  const [appendUploader, setAppendUploader] = useState<boolean>(false)
  const [toDelete, setToDelete] = useState<string[]>([])

  const onUploadComplete = (
    result: UploadResult<Meta, Record<string, never>>
  ) => {
    setFileIds(
      appendUploader
        ? [...fileIds, ...result.successful.map((file) => file.uploadURL)]
        : result.successful.map((file) => file.uploadURL)
    )
    setAppendUploader(false)
  }

  const urlRegex = useMemo(
    () =>
      /^(?:(?:(?:https?|ftp):)?\/\/)(?:\S+(?::\S*)?@)?(?:(?!(?:10|127)(?:\.\d{1,3}){3})(?!(?:169\.254|192\.168)(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)(?:\.(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)*(?:\.(?:[a-z\u00a1-\uffff]{2,})))(?::\d{2,5})?(?:[/?#]\S*)?$/i,
    []
  )

  useEffect(() => {
    setLinkErrors(links.map((link) => link.length > 0 && !urlRegex.test(link)))
  }, [links, urlRegex])

  const onSubmit = (data: FormProject) => {
    const errorsExist = linkErrors.indexOf(true) !== -1
    const errorCount = linkErrors.filter((val) => val).length
    const emptyCount = links.filter((val) => val.trim().length === 0).length

    if (errorsExist) return toast.error(`${errorCount} links invalid`)
    if (emptyCount > 0) return toast.error(`${emptyCount} links empty`)

    batchDelete(toDelete)

    props.onSave(
      {
        title: data.title,
        description: descEditor.getHTML(),
        date: date.toISOString(),
        links: links.filter((link) => link.trim().length > 0),
        images: fileIds,
        tags: selectedTags.map((tag) => tag.id),
        removeTags: props.project?.tags
          .filter((tag) => !selectedTags.some((st) => st.id === tag.id))
          .map((tag) => tag.id),
      },
      props?.project?.id
    )
  }

  const titleRef = useRef<HTMLInputElement>(null)
  useEffect(() => titleRef.current?.focus(), [])

  return (
    <Form<FormProject>
      onSubmit={onSubmit}
      error={props.error}
      className="space-y-2"
    >
      <Label name="title" className="form-control w-full">
        <Label
          name="title"
          className="input input-bordered flex items-center gap-2"
          errorClassName="input input-bordered flex items-center gap-2 input-error"
        >
          <Label
            name="title"
            className="size-4 opacity-70"
            errorClassName="size-4 text-error"
          >
            <Icon path={mdiFormatTitle} />
          </Label>
          <TextField
            name="title"
            ref={titleRef}
            placeholder="Title"
            defaultValue={props.project?.title}
            className="w-full"
            validation={{
              required: {
                value: true,
                message: 'Required',
              },
            }}
          />
        </Label>
        <div className="label">
          <FieldError
            name="title"
            className="text-xs font-semibold text-error"
          />
        </div>
      </Label>

      <RichTextEditor editor={descEditor} />

      <div className="form-control w-full">
        <Label
          name="date"
          className="input input-bordered flex items-center gap-2"
          errorClassName="input input-bordered flex items-center gap-2 input-error"
          onClick={() => setPickerVisible(!pickerVisible)}
        >
          <Label
            name="date"
            className="size-4 opacity-70"
            errorClassName="size-4 text-error"
          >
            <Icon path={mdiCalendar} />
          </Label>
          <TextField
            name="date"
            ref={titleRef}
            placeholder="Date"
            value={format(date, 'PP')}
            className="w-full"
          />
        </Label>
      </div>

      {pickerVisible && (
        <div className="flex justify-center">
          <DatePicker
            date={date}
            setDate={setDate}
            month={month}
            setMonth={setMonth}
          />
        </div>
      )}

      <div className={`${!pickerVisible && 'pt-2'}`}>
        <FormTextList
          name="Links"
          hint="Short links are recommended"
          itemPlaceholder="URL"
          icon={mdiLinkVariant}
          list={links}
          errors={linkErrors}
          setList={setLinks}
        />
      </div>

      <TagsSelectorCell
        selectedTags={selectedTags}
        setSelectedTags={setSelectedTags}
      />

      <div className="py-2 space-y-2">
        <div className="flex space-x-2 justify-between items-center">
          <p className="font-semibold">Images</p>
          {fileIds.length > 0 &&
            (appendUploader ? (
              <button
                className="btn btn-sm"
                type="button"
                onClick={() => setAppendUploader(false)}
              >
                Nevermind
              </button>
            ) : (
              <button
                className="btn btn-sm"
                type="button"
                onClick={() => setAppendUploader(true)}
              >
                Upload More
              </button>
            ))}
        </div>
        {fileIds.length > 0 ? (
          <>
            {!appendUploader &&
              fileIds.map((fileId, i) => (
                <div key={i} className="flex justify-center">
                  <div className="card rounded-xl w-fit image-full image-full-no-overlay">
                    <figure>
                      <img src={fileId} alt={i.toString()} />
                    </figure>
                    <div className="card-body p-2 rounded-xl">
                      <div
                        className={`card-actions rounded-md ${i === 0 ? 'justify-between' : 'justify-end'}`}
                      >
                        {i === 0 && (
                          <div className="btn btn-sm shadow-xl no-animation">
                            Cover Image
                          </div>
                        )}
                        <button
                          type="button"
                          className="btn btn-square btn-sm btn-error shadow-xl"
                          onClick={() => {
                            setToDelete([...toDelete, fileId])
                            setFileIds(fileIds.filter((id) => id !== fileId))
                          }}
                        >
                          <Icon path={mdiDelete} className="size-4" />
                        </button>
                      </div>
                    </div>
                  </div>
                </div>
              ))}
            {appendUploader && (
              <Uploader
                onComplete={onUploadComplete}
                width="auto"
                height="30rem"
                maxFiles={10}
                disabled={props.loading}
              />
            )}
          </>
        ) : (
          <Uploader
            onComplete={onUploadComplete}
            width="auto"
            height="30rem"
            maxFiles={10}
            disabled={props.loading}
          />
        )}
      </div>

      {isAfter(date, today) && (
        <div className="flex justify-center py-2">
          <p>Project will be marked as</p>
          <div className="ml-1 badge badge-info">planned</div>
        </div>
      )}

      <nav className="my-2 flex justify-center space-x-2">
        <Submit
          disabled={props.loading}
          className="btn btn-primary btn-sm uppercase"
        >
          Save
        </Submit>
      </nav>
    </Form>
  )
}

export default ProjectForm
